#include <objmodel.h>

static const struct {
  ADMXRC3_MODEL_TYPE model;
  const TCHAR* pName;
} g_models[] = {
  { ADMXRC3_MODEL_GENERIC, _T("Generic reconfigurable computing card") },
  { ADMXRC3_MODEL_ADMXRC2, _T("ADM-XRC-II") },
  { ADMXRC3_MODEL_ADPEXRC5T, _T("ADPe-XRC-5T") },
  { ADMXRC3_MODEL_ADMXRC6TL, _T("ADM-XRC-6TL") },
  { ADMXRC3_MODEL_ADMXRC6T1, _T("ADM-XRC-6T1") },
  { ADMXRC3_MODEL_ADMXRC6TGE, _T("ADM-XRC-6TGE") },
  { ADMXRC3_MODEL_ADMXRC6TADV8, _T("ADM-XRC-6T-ADV8") },
  { ADMXRC3_MODEL_ADPEXRC6T, _T("ADPE-XRC-6T") },
  { ADMXRC3_MODEL_ADPEXRC6TL, _T("ADPE-XRC-6T-L") },
  { ADMXRC3_MODEL_ADPEXRC6TADV_C, _T("ADPE-XRC-6T-ADV (Controller)") },
  { ADMXRC3_MODEL_ADPEXRC6TADV_T, _T("ADPE-XRC-6T-ADV (Target)") },
  { ADMXRC3_MODEL_ADMXRC7K1, _T("ADM-XRC-7K1") },
  { ADMXRC3_MODEL_ADMXRC6TDA1, _T("ADM-XRC-6T-DA1") },
  { ADMXRC3_MODEL_ADMXRC7V1, _T("ADM-XRC-7V1") },
  { ADMXRC3_MODEL_ADMVPX37V2, _T("ADM-VPX3-7V2") },
  { ADMXRC3_MODEL_ADMXRC6TGEL, _T("ADM-XRC-6TGEL") },
  { ADMXRC3_MODEL_ADMPCIE7V3, _T("ADM-PCIE-7V3") }
};

const TCHAR*
CAdmxrc3Help::getDatatypeString(
  ADMXRC3_DATA_TYPE datatype)
{
  switch (datatype) {
  case ADMXRC3_DATA_BOOL:
    return  _T("boolean");

  case ADMXRC3_DATA_DOUBLE:
    return  _T("double");

  case ADMXRC3_DATA_INT32:
    return  _T("int32");

  case ADMXRC3_DATA_UINT32:
    return  _T("uint32");

  default:
    return NULL;
  }
}

const TCHAR*
CAdmxrc3Help::getFamilyString(
  ADMXRC3_FAMILY_TYPE family)
{
  switch (family) {
  case ADMXRC3_FAMILY_4K:
    return _T("4000 Series");

  case ADMXRC3_FAMILY_VIRTEX:
    return _T("Virtex");

  case ADMXRC3_FAMILY_VIRTEX2:
    return _T("Virtex-II");

  case ADMXRC3_FAMILY_VIRTEX2P:
    return _T("Virtex-II Pro");

  case ADMXRC3_FAMILY_VIRTEX4:
    return _T("Virtex-4");

  case ADMXRC3_FAMILY_VIRTEX5:
    return _T("Virtex-5");

  case ADMXRC3_FAMILY_VIRTEX6:
    return _T("Virtex-6");

  case ADMXRC3_FAMILY_7SERIES:
    return _T("7 Series");

  default:
    return NULL;
  }
}

const TCHAR*
CAdmxrc3Help::getModelName(
  ADMXRC3_MODEL_TYPE modelType)
{
  for (unsigned int i = 0; i < ARRAY_LENGTH(g_models); i++) {
    if (g_models[i].model == modelType) {
      return g_models[i].pName;
    }
  }
  return NULL;
}

const TCHAR*
CAdmxrc3Help::getSubfamilyString(
  ADMXRC3_SUBFAMILY_TYPE subfamily)
{
  switch (subfamily) {
  case ADMXRC3_SUBFAMILY_NONE:
    return _T("No subfamily");

  case ADMXRC3_SUBFAMILY_E:
    return _T("Virtex-E");

  case ADMXRC3_SUBFAMILY_EM:
    return _T("Virtex-EM");

  case ADMXRC3_SUBFAMILY_2PROX:
    return _T("Virtex-II Pro-X");

  case ADMXRC3_SUBFAMILY_4FX:
    return _T("Virtex-4 FX");

  case ADMXRC3_SUBFAMILY_4LX:
    return _T("Virtex-4 LX");

  case ADMXRC3_SUBFAMILY_4SX:
    return _T("Virtex-4 SX");

  case ADMXRC3_SUBFAMILY_5LX:
    return _T("Virtex-5 LX");

  case ADMXRC3_SUBFAMILY_5FXT:
    return _T("Virtex-5 FXT");

  case ADMXRC3_SUBFAMILY_5LXT:
    return _T("Virtex-5 LXT");

  case ADMXRC3_SUBFAMILY_5SXT:
    return _T("Virtex-5 SXT");

  case ADMXRC3_SUBFAMILY_6LX:
    return _T("Virtex-6 LX");

  case ADMXRC3_SUBFAMILY_6HXT:
    return _T("Virtex-6 HXT");

  case ADMXRC3_SUBFAMILY_6LXT:
    return _T("Virtex-6 LXT");

  case ADMXRC3_SUBFAMILY_6SXT:
    return _T("Virtex-6 SXT");

  case ADMXRC3_SUBFAMILY_7AT:
    return _T("Artix-7T");

  case ADMXRC3_SUBFAMILY_7KT:
    return _T("Kintex-7T");

  case ADMXRC3_SUBFAMILY_7VT:
    return _T("Virtex-7T");

  case ADMXRC3_SUBFAMILY_7VXT:
    return _T("Virtex-7XT");

  case ADMXRC3_SUBFAMILY_7VHT:
    return _T("Virtex-7HT");

  default:
    return NULL;
  }
}

const TCHAR*
CAdmxrc3Help::getUnitString(
  ADMXRC3_UNIT_TYPE unit)
{
  switch (unit) {
  case ADMXRC3_UNIT_NONE:
    return _T("no unit");

  case ADMXRC3_UNIT_A:
    return _T("A");

  case ADMXRC3_UNIT_V:
    return _T("V");

  case ADMXRC3_UNIT_C:
    return _T("deg. C");

  case ADMXRC3_UNIT_HZ:
    return _T("Hz");

  case ADMXRC3_UNIT_RPM:
    return _T("RPM");

  case ADMXRC3_UNIT_S:
    return _T("s");

  default:
    return NULL;
  }
}

const TCHAR*
CAdmxrc3Help::getWindowDescription(
  unsigned int index,
  ADMXRC3_MODEL_TYPE model)
{
  switch (model) {
  case ADMXRC3_MODEL_ADMXRC2:
    switch (index) {
    case 0:
      return _T("Target FPGA 0 space");

    case 1:
      return _T("ADM-XRC-II-specific registers");

    case 2:
      return _T("PCI9656 Registers");

    default:
      return NULL;
    }
    break;

  case ADMXRC3_MODEL_ADPEXRC5T:
    switch (index) {
    case 0:
      return _T("Target FPGA 0 prefetchable space");

    case 1:
      return _T("Target FPGA 0 nonprefetchable space");

    case 2:
      return _T("ADPe-XRC-5T-specific registers");

    case 3:
      return _T("ADB3 bridge registers");

    default:
      return NULL;
    }
    break;

  case ADMXRC3_MODEL_ADMXRC6TL:
    switch (index) {
    case 0:
      return _T("Target FPGA 0 prefetchable space");

    case 1:
      return _T("Target FPGA 0 nonprefetchable space");

    case 2:
      return _T("ADM-XRC-6TL-specific registers");

    case 3:
      return _T("ADB3 bridge registers");

    default:
      return NULL;
    }
    break;

  case ADMXRC3_MODEL_ADMXRC6T1:
    switch (index) {
    case 0:
      return _T("Target FPGA 0 prefetchable space");

    case 1:
      return _T("Target FPGA 0 nonprefetchable space");

    case 2:
      return _T("ADM-XRC-6T1-specific registers");

    case 3:
      return _T("ADB3 bridge registers");

    default:
      return NULL;
    }
    break;

  case ADMXRC3_MODEL_ADMXRC6TGE:
  case ADMXRC3_MODEL_ADMXRC6TGEL:
    switch (index) {
    case 0:
      return _T("Target FPGA 0 prefetchable space");

    case 1:
      return _T("Target FPGA 0 nonprefetchable space");

    case 2:
      if (ADMXRC3_MODEL_ADMXRC6TGEL == model) {
        return _T("ADM-XRC-6TGEL-specific registers");
      } else {
        return _T("ADM-XRC-6TGE-specific registers");
      }

    case 3:
      return _T("ADB3 bridge registers");

    default:
      return NULL;
    }
    break;

  case ADMXRC3_MODEL_ADMXRC6TADV8:
    switch (index) {
    case 0:
      return _T("Target FPGA 0 prefetchable space");

    case 1:
      return _T("Target FPGA 0 nonprefetchable space");

    case 2:
      return _T("ADM-XRC-6T-ADV8-specific registers");

    case 3:
      return _T("ADB3 bridge registers");

    default:
      return NULL;
    }
    break;

  case ADMXRC3_MODEL_ADPEXRC6T:
    switch (index) {
    case 0:
      return _T("Target FPGA 0 prefetchable space");

    case 1:
      return _T("Target FPGA 0 nonprefetchable space");

    case 2:
      return _T("ADPE-XRC-6T-specific registers");

    case 3:
      return _T("ADB3 bridge registers");

    default:
      return NULL;
    }
    break;

  case ADMXRC3_MODEL_ADPEXRC6TL:
    switch (index) {
    case 0:
      return _T("Target FPGA 0 prefetchable space");

    case 1:
      return _T("Target FPGA 0 nonprefetchable space");

    case 2:
      return _T("ADPE-XRC-6T-L-specific registers");

    case 3:
      return _T("ADB3 bridge registers");

    default:
      return NULL;
    }
    break;

  case ADMXRC3_MODEL_ADPEXRC6TADV_C:
    switch (index) {
    case 0:
      return _T("Controller FPGA prefetchable space");

    case 1:
      return _T("Controller FPGA nonprefetchable space");

    case 2:
      return _T("ADPE-XRC-6T-ADV-specific registers");

    case 3:
      return _T("ADB3 bridge registers");

    default:
      return NULL;
    }
    break;

  case ADMXRC3_MODEL_ADPEXRC6TADV_T:
    switch (index) {
    case 0:
      return _T("Target FPGA prefetchable space");

    case 1:
      return _T("Target FPGA nonprefetchable space");

    case 2:
      return _T("ADPE-XRC-6T-ADV-specific registers");

    case 3:
      return _T("ADB3 bridge registers");

    default:
      return NULL;
    }
    break;

  case ADMXRC3_MODEL_ADMXRC7K1:
    switch (index) {
    case 0:
      return _T("Target FPGA 0 prefetchable space");

    case 1:
      return _T("Target FPGA 0 nonprefetchable space");

    case 2:
      return _T("ADM-XRC-7K1-specific registers");

    case 3:
      return _T("ADB3 bridge registers");

    default:
      return NULL;
    }
    break;

  case ADMXRC3_MODEL_ADMXRC6TDA1:
    switch (index) {
    case 0:
      return _T("Target FPGA 0 prefetchable space");

    case 1:
      return _T("Target FPGA 0 nonprefetchable space");

    case 2:
      return _T("ADM-XRC-6T-DA1-specific registers");

    case 3:
      return _T("ADB3 bridge registers");

    default:
      return NULL;
    }
    break;

  case ADMXRC3_MODEL_ADMXRC7V1:
    switch (index) {
    case 0:
      return _T("Target FPGA 0 prefetchable space");

    case 1:
      return _T("Target FPGA 0 nonprefetchable space");

    case 2:
      return _T("ADM-XRC-7V1-specific registers");

    case 3:
      return _T("ADB3 bridge registers");

    default:
      return NULL;
    }
    break;

  case ADMXRC3_MODEL_ADMVPX37V2:
    switch (index) {
    case 0:
      return _T("Target FPGA 0 prefetchable space");

    case 1:
      return _T("Target FPGA 0 nonprefetchable space");

    case 2:
      return _T("ADM-VPX3-7V2-specific registers");

    case 3:
      return _T("ADB3 bridge registers");

    default:
      return NULL;
    }
    break;

  case ADMXRC3_MODEL_GENERIC:
  default:
    return NULL;
  }
}
