#pragma once

#include <objmodel.h>
#include "menuitem.h"

class CSysMonWnd : public Gtk::Window, public ISysMonWnd
{
public:
  CSysMonWnd();
  virtual ~CSysMonWnd();
  // ISysMonWnd interface
  bool_t addGraphWnd(void);
  bool_t doClearDeviceErrors();
  bool_t doSelectDeviceIndex(unsigned int deviceIndex);
  bool_t doSelectTab(unsigned int tabIndex);
  const DeviceInfo& getDeviceInfo() const { return m_device; }
  IDeviceInfoWnd* getDeviceInfoWnd() const { return m_pDeviceInfoWnd; }
  IDeviceStatusWnd* getDeviceStatusWnd() const { return m_pDeviceStatusWnd; }
  window_handle_t getNativeWindow() { return *this; }
  IGraphWnd* getGraphWnd(unsigned int index) const {
    ASSERT(index < m_numGraphWnd);
    return m_ppGraphWnd[index];
  }
  unsigned int getNumGraphWnd() const { return m_numGraphWnd; }
  ISensorInfoWnd* getSensorInfoWnd() const { return m_pSensorInfoWnd; }
  unsigned int getUpdatePeriod() const { return m_updatePeriods[m_updatePeriodIndex]; }
  void idle();
  bool_t removeGraphWnd(unsigned int index);
  bool_t removeGraphWnd(IGraphWnd* pGraphWnd);
  void requestQuit();

protected:
  // Following members define main window default size, in pixels
  static const int m_defaultWidth = 640;
  static const int m_defaultHeight = 480;
  static const int m_minWidth = 520;
  static const int m_minHeight = 360;

  static const int m_updateTimerNumber = 0;

  // Sensor read periods, selected m_freqSelectCtl
  static const unsigned int m_updatePeriods[];

  enum TabIndex {
    TabDevoceInfo = 0,
    TabSensorInfo = 1,
    TabSensorReadings = 2
  };
  unsigned int m_deviceIndex;
  unsigned int m_numGraphWnd;
  int m_updatePeriodIndex;
  TabIndex m_currentTab;
  DeviceInfo m_device;
  IGraphWnd** m_ppGraphWnd;
  IDeviceInfoWnd* m_pDeviceInfoWnd;
  IDeviceStatusWnd* m_pDeviceStatusWnd;
  ISensorInfoWnd* m_pSensorInfoWnd;
  bool m_bLogging;
  bool m_bLogAppend;
  std::string m_logFilename;
  timestamp_t m_logTimestamp;

protected:
  Glib::RefPtr<Gdk::Pixbuf> m_iconPixbuf;
  Glib::RefPtr<Gtk::StatusIcon> m_trayIcon;
  Gtk::Menu m_trayMenu;
  Gtk::MenuItem m_trayMenuRestore;
  Gtk::MenuItem m_trayMenuSeparator1;
  Gtk::MenuItem m_trayMenuStartNewLog;
  Gtk::MenuItem m_trayMenuAppendToLog;
  Gtk::MenuItem m_trayMenuStopLog;
  Gtk::MenuItem m_trayMenuSeparator2;
  Gtk::MenuItem m_trayMenuExit;
  Gtk::VBox m_vBox1Ctl;
  Gtk::Notebook m_notebookCtl;
  Gtk::Label m_notebookLabel1Ctl;
  Gtk::Label m_notebookLabel2Ctl;
  Gtk::Label m_notebookLabel3Ctl;
  Gtk::Label m_notebookLabel4Ctl;
  Gtk::HBox m_hBox1Ctl;
  Gtk::HBox m_hBox2Ctl;
  Gtk::HBox m_hBoxCtl;
  Gtk::Table m_tableCtl;
  Gtk::Label m_deviceLabelCtl;
  Gtk::ComboBoxText m_deviceSelectCtl;
  Gtk::Label m_freqLabelCtl;
  Gtk::ComboBoxText m_freqSelectCtl;
  Gtk::Button m_actionCtl;
  Gtk::Menu m_actionMenuCtl;
  Gtk::MenuItem m_actionMenuStartNewLog;
  Gtk::MenuItem m_actionMenuAppendToLog;
  Gtk::MenuItem m_actionMenuStopLog;
  Gtk::SeparatorMenuItem m_actionMenuSeparator;
  Gtk::MenuItem m_actionMenuAbout;
  sigc::connection m_timer;
  sigc::slot<bool> m_timerSlot;
  int m_timerNumber;
  bool_t m_bTimerRunning;
  bool_t m_bSelectDeviceFromControl;

protected:
  void enumerateDevices(ADMXRC3_CARD_INFOEX* pCardInfoEx);
  void onAbout();
  void onAction();
  void onStartNewLog();
  void onAppendToLog();
  void onStopLog();
  bool onDelete(GdkEventAny* pEventInfo);
  void onPopupTrayMenu(guint button, guint32 activateTime);
  void onSelectDevice();
  void onSelectUpdatePeriod();
  bool onTimer(int timerNumber);
  void onTrayActivate();
  void onTrayMenuExit();
  void onTrayMenuRestore();
  bool onWindowStateChanged(GdkEventWindowState* pEventInfo);
  void readSensors(timestamp_t timestamp);
  void setDevice(unsigned int deviceIndex, bool_t bFromControl = FALSE);
  void setUpdatePeriod(unsigned int index);
  void setNumGraphWnd(unsigned int n);
  void startLogging(const std::string& filename, bool bAppend);
  void stopLogging(bool bQuiet);
  void emitLogHeader(FILE* pFile, const DeviceInfo& devInfo, timestamp_t& timestamp);
};
