#include "stdafx.h"
#include "afxwinappex.h"

#ifdef _DEBUG
# define new DEBUG_NEW
#endif

#include <platform.h>
#include <objmodel.h>

#include "control_id.h"
#include "devicestatuswnd.h"
#include "uachelp.h"

IMPLEMENT_DYNAMIC(CDeviceStatusWnd, CWnd)

BEGIN_MESSAGE_MAP(CDeviceStatusWnd, CWnd)
  ON_WM_CREATE()
  ON_WM_SIZE()
  ON_WM_CTLCOLOR()
  ON_COMMAND(IDCTL_CLEARERRORS, OnButtonClearErrors)
END_MESSAGE_MAP()

static const float g_fMinTotalWidth = 260.0;
static const float g_fColumnResource = 0.65f;
static const float g_fColumnValue = 0.35f;

// -------------------------------------------------------------------------
// Start of IDeviceStatusWnd interface
// -------------------------------------------------------------------------

void
CDeviceStatusWnd::deviceChanged()
{
  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  unsigned int j;
  CString str;

  if (!m_bFunctionalityAvailable) {
    return;
  }

  m_decListCtl.SetRedraw(FALSE);
  m_decListCtl.DeleteAllItems();
  j = 0;
  m_decListCtl.InsertItem(j, _T("Error Type"));
  m_decListCtl.SetItem(j, 1, LVIF_TEXT, _T("Value"), 0, 0, 0, (LPARAM)0);
  j++;
  m_decListCtl.InsertItem(j, _T(""));
  j++;
  if (devInfo.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE) {
    for (unsigned int i = 0; i < devInfo.m_deviceInfo.NumWindow; i++, j++) {
      str.Format(_T("Direct Slave Window %u timeout"), i);
      m_decListCtl.InsertItem(j, str);
    }
    for (unsigned int i = 0; i < devInfo.m_deviceInfo.NumDmaChannel; i++, j++) {
      str.Format(_T("DMA engine %u bus timeout"), i);
      m_decListCtl.InsertItem(j, str);
    }
    m_decListCtl.InsertItem(j, _T("Direct Master bus timeout"));
    j++;
    m_decListCtl.InsertItem(j, _T("Direct Master bad transaction"));
    j++;
  }
  m_decListCtl.SetRedraw(TRUE);

  m_clearCtl.EnableWindow((devInfo.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE && m_bInAdminGroup) ? TRUE : FALSE);

  update();
}

bool_t
CDeviceStatusWnd::doClearDeviceErrors()
{
  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  ADMXRC3_HANDLE hDevice = ADMXRC3_HANDLE_INVALID_VALUE;
  ADMXRC3_STATUS status;
  CString msg;

  if (!m_bFunctionalityAvailable) {
    msg.Format(_T("Fuctionality not available in driver"));
    MessageBox(msg, _T("Failed to clear device errors"), MB_ICONEXCLAMATION | MB_OK);
    return FALSE;
  }

  if (devInfo.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE) {
    status = ADMXRC3_OpenEx(devInfo.m_deviceIndex, FALSE, 0, &hDevice);
    if (ADMXRC3_SUCCESS != status) {
      msg.Format(_T("Failed to reopen device in active mode\n\nIndex=%u Status=0x%x(%s)\n"), devInfo.m_deviceIndex, status, ADMXRC3_GetStatusString(status, TRUE));
      MessageBox(msg, _T("Failed to clear device errors"), MB_ICONEXCLAMATION | MB_OK);
      return FALSE;
    }

    status = m_pAdmxrc3ClearDeviceErrors(hDevice);
    if (ADMXRC3_SUCCESS != status) {
      msg.Format(_T("Call to ADMXRC3_ClearDeviceErrors failed\n\nIndex=%u Status=0x%x(%s)\n"), devInfo.m_deviceIndex, status, ADMXRC3_GetStatusString(status, TRUE));
      MessageBox(msg, _T("Failed to clear device errors"), MB_ICONEXCLAMATION | MB_OK);
      ADMXRC3_Close(hDevice);
      return FALSE;
    }

    ADMXRC3_Close(hDevice);

    update();

    return TRUE;
  } else {
    msg.Format(_T("No device selected\n"));
    MessageBox(msg, _T("Failed to clear device errors"), MB_ICONEXCLAMATION | MB_OK);
    return FALSE;
  }
}

void
CDeviceStatusWnd::numDeviceChanged()
{
  unsigned int numDevice = m_pParent->getDeviceInfo().m_numDevice;

  if ((m_numDevice == 0) == (numDevice == 0)) {
    return;
  }

  m_numDevice = numDevice;
}

void
CDeviceStatusWnd::update()
{
  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  ADMXRC3_DEVICE_STATUS devStatus;
  ADMXRC3_STATUS status;
  unsigned int j;
  CString str;

  if (!m_bFunctionalityAvailable) {
    return;
  }

  if (devInfo.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE) {
    m_rawListCtl.SetRedraw(FALSE);
    m_decListCtl.SetRedraw(FALSE);
    status = m_pAdmxrc3GetDeviceStatus(devInfo.m_hDevice, 0, &devStatus, NULL);
    if (ADMXRC3_SUCCESS != status) {
      j = 2;
      for (unsigned int i = 0; i < 5; i++, j++) {
        m_rawListCtl.SetItem(j, 1, LVIF_TEXT, _T("?"), 0, 0, 0, (LPARAM)0);
      }
      j = 2;
      for (unsigned int i = 0; i < devInfo.m_deviceInfo.NumWindow; i++, j++) {
        m_decListCtl.SetItem(j, 1, LVIF_TEXT, _T("?"), 0, 0, 0, (LPARAM)0);
      }
      for (unsigned int i = 0; i < devInfo.m_deviceInfo.NumDmaChannel; i++, j++) {
        m_decListCtl.SetItem(j, 1, LVIF_TEXT, _T("?"), 0, 0, 0, (LPARAM)0);
      }
      m_decListCtl.SetItem(j, 1, LVIF_TEXT, _T("?"), 0, 0, 0, (LPARAM)0);
      j++;
      m_decListCtl.SetItem(j, 1, LVIF_TEXT, _T("?"), 0, 0, 0, (LPARAM)0);
      j++;
    } else {
      j = 2;
      str.Format(_T("0x%08lx"), (unsigned long)devStatus.Window);
      m_rawListCtl.SetItem(j++, 1, LVIF_TEXT, str, 0, 0, 0, (LPARAM)0);
      str.Format(_T("0x%08lx"), (unsigned long)devStatus.DMA);
      m_rawListCtl.SetItem(j++, 1, LVIF_TEXT, str, 0, 0, 0, (LPARAM)0);
      str.Format(_T("0x%08lx"), (unsigned long)devStatus.DirectMaster);
      m_rawListCtl.SetItem(j++, 1, LVIF_TEXT, str, 0, 0, 0, (LPARAM)0);
      str.Format(_T("0x%08lx"), (unsigned long)devStatus.ModelLow);
      m_rawListCtl.SetItem(j++, 1, LVIF_TEXT, str, 0, 0, 0, (LPARAM)0);
      str.Format(_T("0x%08lx"), (unsigned long)devStatus.ModelHigh);
      m_rawListCtl.SetItem(j++, 1, LVIF_TEXT, str, 0, 0, 0, (LPARAM)0);
      j = 2;
      for (unsigned int i = 0; i < devInfo.m_deviceInfo.NumWindow; i++, j++) {
        str.Format(_T("%u"), (ADMXRC3_DS_LOCAL_TIMEOUT(i) & devStatus.Window) ? 1U : 0U);
        m_decListCtl.SetItem(j, 1, LVIF_TEXT, str, 0, 0, 0, (LPARAM)0);
      }
      for (unsigned int i = 0; i < devInfo.m_deviceInfo.NumDmaChannel; i++, j++) {
        str.Format(_T("%u"), (ADMXRC3_DMA_BUS_TIMEOUT(i) & devStatus.DMA) ? 1U : 0U);
        m_decListCtl.SetItem(j, 1, LVIF_TEXT, str, 0, 0, 0, (LPARAM)0);
      }
      str.Format(_T("%u"), (ADMXRC3_DM_BUS_TIMEOUT & devStatus.DirectMaster) ? 1U : 0U);
      m_decListCtl.SetItem(j, 1, LVIF_TEXT, str, 0, 0, 0, (LPARAM)0);
      j++;
      str.Format(_T("%u"), (ADMXRC3_DM_BAD_TRANSACTION & devStatus.DirectMaster) ? 1U : 0U);
      m_decListCtl.SetItem(j, 1, LVIF_TEXT, str, 0, 0, 0, (LPARAM)0);
      j++;
    }
    m_rawListCtl.SetRedraw(TRUE);
    m_decListCtl.SetRedraw(TRUE);
  }
}

// -------------------------------------------------------------------------
// End of IDeviceStatusWnd interface
// -------------------------------------------------------------------------

CDeviceStatusWnd::CDeviceStatusWnd(
  ISysMonWnd* pParent,
  bool_t bVisible)
{
  const TCHAR* pWndClass = NULL;
  bool_t bSuccess;

  m_pParent = pParent;
  m_numDevice = 0;
  m_bInAdminGroup = FALSE;

  m_bgBrush.CreateSysColorBrush(COLOR_BTNFACE);
  m_font.CreateStockObject(DEFAULT_GUI_FONT);

  pWndClass = AfxRegisterWndClass(0, ::LoadCursor(NULL, IDC_ARROW), NULL, NULL);
  bSuccess = CreateEx(WS_EX_CONTROLPARENT, pWndClass, _T(""), WS_CHILD | (bVisible ? WS_VISIBLE : 0), 0, 0, 1, 1, m_pParent->getNativeWindow(), NULL, NULL);
  ASSERT(bSuccess);
}

CDeviceStatusWnd::~CDeviceStatusWnd()
{
}

int
CDeviceStatusWnd::OnCreate(
  LPCREATESTRUCT lpCreateStruct)
{
  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  RECT r = { 0, 0, 0, 0 };

  if (CWnd::OnCreate(lpCreateStruct) == -1) {
    return -1;
  }

  /*
  ** We dynamically get the entry points for ADMXRC3_GetDeviceStatus and
  ** ADMXRC3_ClearDeviceErrors so that if the ADMXRC3 runtime installed on
  ** the system happens to predate the addition of these functions to the
  ** ADMXRC3 API, we can gracefully disable the corresponding functionality
  ** in this application.
  */
  m_bFunctionalityAvailable = FALSE; /* Assume FALSE, can set true later */
#if _DEBUG
  m_hAdmxrc3Dll = GetModuleHandle(_T("admxrc3d"));
#else
  m_hAdmxrc3Dll = GetModuleHandle(_T("admxrc3"));
#endif
  if (NULL != m_hAdmxrc3Dll) {
    m_pAdmxrc3GetDeviceStatus = (ADMXRC3_GETDEVICESTATUS*)GetProcAddress(m_hAdmxrc3Dll, "ADMXRC3_GetDeviceStatus");
    m_pAdmxrc3ClearDeviceErrors = (ADMXRC3_CLEARDEVICEERRORS*)GetProcAddress(m_hAdmxrc3Dll, "ADMXRC3_ClearDeviceErrors");
    if (NULL != m_pAdmxrc3GetDeviceStatus && NULL != m_pAdmxrc3ClearDeviceErrors) {
      m_bFunctionalityAvailable = TRUE;
    }
  }

  if (m_bFunctionalityAvailable) {
    m_rawLabelCtl.Create(_T("Device status"), WS_VISIBLE | WS_CHILD | SS_LEFT, r, this, (UINT)IDC_STATIC);
    m_rawLabelCtl.SetFont(&m_font);

    m_rawListCtl.CreateEx(WS_EX_STATICEDGE, WS_CHILD | WS_VISIBLE | WS_TABSTOP | LVS_REPORT | LVS_NOCOLUMNHEADER, r, this, IDCTL_RAWDEVICESTATUS_LIST);
    m_rawListCtl.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_INFOTIP);
    m_rawListCtl.SetFont(&m_font);
    m_rawListCtl.InsertColumn(0, _T("Field"), LVCFMT_LEFT, (int)(g_fMinTotalWidth * g_fColumnResource), -1);
    m_rawListCtl.InsertColumn(1, _T("Value"), LVCFMT_LEFT, (int)(g_fMinTotalWidth * g_fColumnValue), -1);
    m_rawListCtl.InsertItem(0, _T("Field"));
    m_rawListCtl.SetItem(0, 1, LVIF_TEXT, _T("Value"), 0, 0, 0, (LPARAM)0);
    m_rawListCtl.InsertItem(1, _T(""));
    m_rawListCtl.InsertItem(2, _T("Window"));
    m_rawListCtl.InsertItem(3, _T("DMA"));
    m_rawListCtl.InsertItem(4, _T("DirectMaster"));
    m_rawListCtl.InsertItem(5, _T("ModelLow"));
    m_rawListCtl.InsertItem(5, _T("ModelHigh"));

    m_decLabelCtl.Create(_T("Decoded device status"), WS_VISIBLE | WS_CHILD | SS_LEFT, r, this, (UINT)IDC_STATIC);
    m_decLabelCtl.SetFont(&m_font);

    m_decListCtl.CreateEx(WS_EX_STATICEDGE, WS_CHILD | WS_VISIBLE | WS_TABSTOP | LVS_REPORT | LVS_NOCOLUMNHEADER, r, this, IDCTL_DECDEVICESTATUS_LIST);
    m_decListCtl.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_INFOTIP);
    m_decListCtl.SetFont(&m_font);
    m_decListCtl.InsertColumn(0, _T("Error Type"), LVCFMT_LEFT, (int)(g_fMinTotalWidth * g_fColumnResource), -1);
    m_decListCtl.InsertColumn(1, _T("Value"), LVCFMT_LEFT, (int)(g_fMinTotalWidth * g_fColumnValue), -1);

    /* Create the "Clear Errors" button */
    m_clearCtl.Create(_T("Clear Errors"), WS_VISIBLE | WS_CHILD | WS_TABSTOP, r, this, IDCTL_CLEARERRORS);
    m_clearCtl.SetFont(&m_font);
    m_clearCtl.EnableWindow(FALSE);

    m_clearLabelCtl.Create(_T(""), WS_CHILD | SS_LEFT | WS_VISIBLE, r, this, (UINT)IDC_STATIC);
    m_clearLabelCtl.SetFont(&m_font);

    /* Attempt to add UAC shield; will fail if < Vista, but we don't care. */
  #define BCM_SETSHIELD (BCM_FIRST + 0x000C)
    ::SendMessage(m_clearCtl.m_hWnd, (UINT)BCM_SETSHIELD, (WPARAM)0, (LPARAM)TRUE);

    /* Figure out if we are in Administrators group, and enable the "Clear Errors" button if yes. */
    try {
      m_bInAdminGroup = IsUserInAdminGroup();
      if (!m_bInAdminGroup) {
        m_clearLabelCtl.SetWindowText(_T("Function unavailable because not in Administrators group"));
      } else {
        if (devInfo.m_hDevice != ADMXRC3_HANDLE_INVALID_VALUE) {
          m_clearCtl.EnableWindow(TRUE);
        }
      }
    } catch (DWORD dwError) {
      CString str;

      str.Format(_T("Function unavailable because unable to determine if in Administrators group, error=0x%x\n"), dwError);
      m_clearLabelCtl.SetWindowText(str);
    }
  } else {
    m_notAvailableCtl.Create(_T("Functionality not available in driver"), WS_VISIBLE | WS_CHILD | SS_CENTER | SS_CENTERIMAGE, r, this, (UINT)IDC_STATIC);
    m_notAvailableCtl.SetFont(&m_font);
  }

  return 0;
}


void
CDeviceStatusWnd::OnSize(
  UINT nType,
  int cx,
  int cy)
{
  int centrex = cx >> 1;
  int w1 = centrex - 20;
  int w2 = cx - w1;

  CWnd::OnSize(nType, cx, cy);

  if (m_bFunctionalityAvailable) {
    m_rawLabelCtl.MoveWindow(0, 0, w1, 18);
    m_rawListCtl.MoveWindow(0, 20, w1, cy - 52);
    m_decLabelCtl.MoveWindow(centrex + 20, 0, cx - w2 - 1, 18);
    m_decListCtl.MoveWindow(centrex + 20, 20, cx - w2 - 1, cy - 52);
    m_clearCtl.MoveWindow(0, cy - 23, 90, 23);
    m_clearLabelCtl.MoveWindow(100, cy - 18, cx - 100, 18);
  } else {
    m_notAvailableCtl.MoveWindow(0, 0, cx, cy);
  }
}

HBRUSH
CDeviceStatusWnd::OnCtlColor(
  CDC* pDC,
  CWnd* pWnd,
  UINT nCtlColor)
{
  if (m_bFunctionalityAvailable && (pWnd == &m_rawLabelCtl || pWnd == &m_decLabelCtl || pWnd == &m_clearLabelCtl)) {
    pDC->SetBkMode(TRANSPARENT);
    return (HBRUSH)GetStockObject(HOLLOW_BRUSH);
  }

  return CWnd::OnCtlColor(pDC, pWnd, nCtlColor);
}

void
CDeviceStatusWnd::OnButtonClearErrors()
{
  const DeviceInfo& devInfo = m_pParent->getDeviceInfo();
  BOOL bIsRunAsAdmin;
  CString msg, param;
  TCHAR szPath[MAX_PATH];
  SHELLEXECUTEINFO sei;

  try {
    bIsRunAsAdmin = IsRunAsAdmin(); 
  } catch (DWORD dwError) {
    msg.Format(_T("Error code 0x%x\n"), dwError);
    MessageBox(msg, _T("Failed to determine if running as Administrator"), MB_ICONEXCLAMATION | MB_OK);
    return;
  }

  if (bIsRunAsAdmin) {
    doClearDeviceErrors();
  } else {
    /* Re-launch ourself elevated. */
    if (GetModuleFileName(NULL, szPath, ARRAYSIZE(szPath))) {
      ZeroMemory(&sei, sizeof(sei));
      sei.cbSize = sizeof(sei);
      sei.lpVerb = _T("runas");
      sei.lpFile = szPath;
      /*
      ** This command line causes (in this order):
      ** 1. The current device to be selected.
      ** 2. The "Device Status" tab to be selected.
      ** 3. Device errors to be cleared as if the "Clear Errors" button were clicked.
      */
      param.Format(_T("-verb sel_dev_idx %u -verb sel_tab 3 -verb clr_dev_err"), devInfo.m_deviceIndex);
      sei.lpParameters = param;
      sei.hwnd = m_hWnd;
      sei.nShow = SW_NORMAL;
      if (!ShellExecuteEx(&sei)) {
        DWORD dwError = GetLastError();

        if (dwError == ERROR_CANCELLED) {
          /* The user refused the elevation; do nothing more. */
        } else {
          msg.Format(_T("File='%s'\nError=0x%lx"), szPath, (unsigned long)dwError);
          MessageBox(msg, _T("Failed to re-launch as Administrator"), MB_ICONEXCLAMATION | MB_OK);
        }
      } else {
        m_pParent->requestQuit();
      } 
    } else {
      msg.Format(_T("Win32 error 0x%lx\n"), (unsigned long)GetLastError());
      MessageBox(msg, _T("Failed to get own executable filename"), MB_ICONEXCLAMATION | MB_OK);
    }
  }
}
