#include "stdafx.h"
#include "afxwinappex.h"

#ifdef _DEBUG
# define new DEBUG_NEW
#endif

#include <platform.h>
#include <objmodel.h>

#include "graphwnd.h"
#include "control_id.h"

IMPLEMENT_DYNAMIC(CGraphWnd, CWnd)

BEGIN_MESSAGE_MAP(CGraphWnd, CWnd)
  ON_WM_CREATE()
  ON_WM_DESTROY()
  ON_WM_SIZE()
END_MESSAGE_MAP()

#if !defined(SHOW_SCROLLBAR)
# define SHOW_SCROLLBAR 0 /* Scrollbar not fully implemented yet */
#endif

// -------------------------------------------------------------------------
// Start of IGraphWnd interface
// -------------------------------------------------------------------------

void
CGraphWnd::deviceChanged()
{
  m_pScope->deviceChanged();
  m_pToolbar->deviceChanged();
}

void
CGraphWnd::update(timestamp_t timestamp)
{
  if (!m_pToolbar->getPaused()) {
    m_currentTimestamp = timestamp - m_startTimestamp;
    m_pScope->update(m_currentTimestamp);
  }
}

void
CGraphWnd::setIndex(
  unsigned int index)
{
  m_index = index;
  m_pToolbar->setIndex(index);
}

void
CGraphWnd::started()
{
  LARGE_INTEGER count, frequency;

  QueryPerformanceCounter(&count);
  QueryPerformanceFrequency(&frequency);
  m_startTimestamp = (timestamp_t)((double)count.QuadPart * 1000.0 / (double)frequency.QuadPart);
  m_currentTimestamp = TIMESTAMP_INVALID;
}

void
CGraphWnd::viewChanged(
  timestamp_t interval,
  timestamp_t minShow,
  timestamp_t maxShow)
{
  if (NULL != m_pScrollbar) {
    SCROLLINFO sinfo;
    sinfo.cbSize = sizeof(SCROLLINFO);
    sinfo.fMask = SIF_ALL;
    sinfo.nMin = 0;
    sinfo.nMax = (int)interval;
    sinfo.nPage = (UINT)(maxShow - minShow);
    sinfo.nPos = (int)minShow;
    m_pScrollbar->SetScrollInfo(&sinfo, TRUE);
  }
}

// -------------------------------------------------------------------------
// End of IGraphWnd interface
// -------------------------------------------------------------------------

CGraphWnd::CGraphWnd(
  ISysMonWnd* pParent,
  unsigned int index,
  bool_t bVisible)
{
  const TCHAR* pWndClass = NULL;
  bool_t bSuccess;

  m_pParent = pParent;
  m_pToolbar = NULL;
  m_pScope = NULL;
  m_index = index;
  m_pScrollbar = NULL;

  pWndClass = AfxRegisterWndClass(0, ::LoadCursor(NULL, IDC_ARROW), NULL, NULL);
  bSuccess = CreateEx(WS_EX_STATICEDGE | WS_EX_CONTROLPARENT, pWndClass, _T(""), WS_CHILD | (bVisible ? WS_VISIBLE : 0), 0, 0, 1, 1, m_pParent->getNativeWindow(), NULL, NULL);
  ASSERT(bSuccess);
}

CGraphWnd::~CGraphWnd()
{
}

void
CGraphWnd::OnDestroy()
{
  CWnd::OnDestroy();

  if (NULL != m_pToolbar) {
    m_pToolbar->getNativeControl().DestroyWindow();
    delete m_pToolbar;
    m_pToolbar = NULL;
  }
  if (NULL != m_pScope) {
    m_pScope->getNativeControl().DestroyWindow();
    delete m_pScope;
    m_pScope = NULL;
  }
  if (NULL != m_pScrollbar) {
    m_pScrollbar->DestroyWindow();
    delete m_pScrollbar;
    m_pScrollbar = NULL;
  }
}

int
CGraphWnd::OnCreate(
  LPCREATESTRUCT lpCreateStruct)
{
  if (CWnd::OnCreate(lpCreateStruct) == -1) {
    return -1;
  }

  RECT r;
  GetClientRect(&r);
  m_pToolbar = getFactory().createGraphWndToolbar(this, m_index);
  m_pScope = getFactory().createScopeWnd(this);
#if SHOW_SCROLLBAR
  m_pScrollbar = new CScrollBar();
#endif
  if (NULL != m_pScrollbar) {
    m_pScrollbar->Create(SBS_HORZ | SBS_BOTTOMALIGN | WS_VISIBLE | WS_CHILD, r, this, IDCTL_SCOPE_SCROLLBAR);
    m_pScrollbar->SetScrollRange(0, 1, FALSE);
    SCROLLINFO sinfo;
    sinfo.cbSize = sizeof(SCROLLINFO);
    sinfo.fMask = SIF_ALL;
    sinfo.nMin = 0;
    sinfo.nMax = 0;
    sinfo.nPage = 1;
    sinfo.nPos = 0;
    m_pScrollbar->SetScrollInfo(&sinfo, TRUE);
  }
  return 0;
}

void
CGraphWnd::OnSize(
  UINT nType,
  int cx,
  int cy)
{
  CWnd::OnSize(nType, cx, cy);

  m_pToolbar->getNativeControl().MoveWindow(0, 0, cx, 32);
  if (NULL != m_pScrollbar) {
    CRect sbr;
    m_pScrollbar->GetClientRect(sbr);
    m_pScope->getNativeControl().MoveWindow(0, 32, cx, cy - (32 + sbr.Height()));
    m_pScrollbar->MoveWindow(0, cy - sbr.Height(), cx, sbr.Height());
  } else {
    m_pScope->getNativeControl().MoveWindow(0, 32, cx, cy - 32);
  }
}
